#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/i2c.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/pm/device.h>
#include <zephyr/sys/poweroff.h>
#include <zephyr/sys/util.h>

#include "retained.h"

#include <inttypes.h>
#include <stdio.h>

#include "lis3dh.h"

#define I2C_NODE DT_NODELABEL(lis3dh)
static const struct i2c_dt_spec dev_i2c = I2C_DT_SPEC_GET(I2C_NODE);

#define INT1_NODE DT_ALIAS(int1)
static const struct gpio_dt_spec interrupt1 = GPIO_DT_SPEC_GET(INT1_NODE, gpios);

int main(void)
{
	int ret;
	/* Get the Console */
	const struct device *const cons = DEVICE_DT_GET(DT_CHOSEN(zephyr_console));
	if (!device_is_ready(cons))
	{
		printf("%s: device not ready.\n", cons->name);
		return 0;
	}

	if (IS_ENABLED(CONFIG_APP_RETENTION))
	{
		bool retained_ok = retained_validate();

		/* Increment for this boot attempt and update. */
		retained.boots += 1;
		retained_update();

		printf("Retained data: %s\n", retained_ok ? "valid" : "INVALID");
		printf("Boot count: %u\n", retained.boots);
		printf("Off count: %u\n", retained.off_count);
		printf("Active Ticks: %" PRIu64 "\n", retained.uptime_sum);
	}
	else
	{
		printf("Retained data not supported\n");
	}

	/* Initialize the Interrupt Pins of LIS3DH */
	ret = gpio_pin_configure_dt(&interrupt1, GPIO_INPUT);
	if (ret < 0)
	{
		return -1;
	}
	ret = gpio_pin_interrupt_configure_dt(&interrupt1, GPIO_INT_EDGE_TO_ACTIVE);
	if (ret < 0)
	{
		printf("Could not configure sw0 GPIO interrupt");
		return -1;
	}

	/* Initialize I2C Communication */

	if (!device_is_ready(dev_i2c.bus))
	{
		printf("I2C bus %s is not ready!\n\r", dev_i2c.bus->name);
		return -1;
	}

	/* Read WHO_I_AM register */
	uint8_t devID = 0x33;
	uint8_t reg = 0;
	uint8_t sensor_regs = LIS3DH_REG_WHOAMI;
	ret = i2c_write_read_dt(&dev_i2c, &sensor_regs, 1, &reg, 1);
	if (ret != 0)
	{
		printf("Failed to write/read I2C device address %x at Reg. %x \r\n", dev_i2c.addr, sensor_regs);
	}
	printf("%x", reg);
	if (devID != reg)
	{
		return -1;
	}

	/* Disable ADC and TEMP sensors */
	uint8_t config[2] = {LIS3DH_REG_TEMPCFG, 0x00};
	ret = i2c_write_dt(&dev_i2c, config, sizeof(config));
	if (ret != 0)
	{
		printf("Failed to write to I2C device address %x at Reg. %x \n", dev_i2c.addr, config[0]);
		return -1;
	}

	/* Turn on the Sensor and enable all axis */
	config[0] = LIS3DH_REG_CTRL1;
	config[1] = 0x47;
	ret = i2c_write_dt(&dev_i2c, config, sizeof(config));
	if (ret != 0)
	{
		printf("Failed to write to I2C device address %x at Reg. %x \n", dev_i2c.addr, config[0]);
		return -1;
	}

	/* Enable High Pass Filter on Interrupt 1 */
	config[0] = LIS3DH_REG_CTRL2;
	config[1] = 0x09;
	ret = i2c_write_dt(&dev_i2c, config, sizeof(config));
	if (ret != 0)
	{
		printf("Failed to write to I2C device address %x at Reg. %x \n", dev_i2c.addr, config[0]);
		return -1;
	}

	/* Enable Interrupt Activity 1 driven to INT1 pad */
	config[0] = LIS3DH_REG_CTRL3;
	config[1] = 0x40;
	ret = i2c_write_dt(&dev_i2c, config, sizeof(config));
	if (ret != 0)
	{
		printf("Failed to write to I2C device address %x at Reg. %x \n", dev_i2c.addr, config[0]);
		return -1;
	}

	/* Set the Full Scale to +-2g */
	config[0] = LIS3DH_REG_CTRL4;
	config[1] = 0x00;
	ret = i2c_write_dt(&dev_i2c, config, sizeof(config));
	if (ret != 0)
	{
		printf("Failed to write to I2C device address %x at Reg. %x \n", dev_i2c.addr, config[0]);
		return -1;
	}

	/* Do not Latch Interrupt Pin 1 */
	config[0] = LIS3DH_REG_CTRL5;
	config[1] = 0x00;
	ret = i2c_write_dt(&dev_i2c, config, sizeof(config));
	if (ret != 0)
	{
		printf("Failed to write to I2C device address %x at Reg. %x \n", dev_i2c.addr, config[0]);
		return -1;
	}

	/* Set the Interrupt Threshold */
	config[0] = LIS3DH_REG_INT1THS;
	config[1] = 0x10;
	ret = i2c_write_dt(&dev_i2c, config, sizeof(config));
	if (ret != 0)
	{
		printf("Failed to write to I2C device address %x at Reg. %x \n", dev_i2c.addr, config[0]);
		return -1;
	}

	/* Set the Interrupt Duration */
	config[0] = LIS3DH_REG_INT1DUR;
	config[1] = 0x01;
	ret = i2c_write_dt(&dev_i2c, config, sizeof(config));
	if (ret != 0)
	{
		printf("Failed to write to I2C device address %x at Reg. %x \n", dev_i2c.addr, config[0]);
		return -1;
	}

	/* Configure Wake UP Event */
	config[0] = LIS3DH_REG_INT1CFG;
	config[1] = 0x2A;
	ret = i2c_write_dt(&dev_i2c, config, sizeof(config));
	if (ret != 0)
	{
		printf("Failed to write to I2C device address %x at Reg. %x \n", dev_i2c.addr, config[0]);
		return -1;
	}

	/* Read reference to force HP filter to current acceleration value */
	reg = 0;
	sensor_regs = LIS3DH_REG_REFERENCE;
	ret = i2c_write_read_dt(&dev_i2c, &sensor_regs, 1, &reg, 1);
	if (ret != 0)
	{
		printf("Failed to write/read I2C device address %x at Reg. %x \r\n", dev_i2c.addr, sensor_regs);
	}

	ret = pm_device_action_run(cons, PM_DEVICE_ACTION_SUSPEND);
	if (ret < 0)
	{
		printf("Could not suspend console");
		return 0;
	}
	if (IS_ENABLED(CONFIG_APP_RETENTION))
	{
		/* Update the retained state */
		retained.off_count += 1;
		retained_update();
	}
	sys_poweroff();
	return 0;
}